package jdk.internal.org.objectweb.asm.util;

import jdk.internal.org.objectweb.asm.Type;
import java.util.HashMap;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Attribute;
import jdk.internal.org.objectweb.asm.TypePath;
import java.io.PrintWriter;
import java.io.FileInputStream;
import jdk.internal.org.objectweb.asm.ClassReader;
import jdk.internal.org.objectweb.asm.Label;
import java.util.Map;

public class ASMifier extends Printer
{
    protected final String name;
    protected final int id;
    protected Map<Label, String> labelNames;
    private static final int ACCESS_CLASS = 262144;
    private static final int ACCESS_FIELD = 524288;
    private static final int ACCESS_INNER = 1048576;
    
    public ASMifier() {
        this(327680, "cw", 0);
        if (this.getClass() != ASMifier.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    protected ASMifier(int n, String name, int id) {
        super(n);
        this.name = name;
        this.id = id;
    }
    
    public static void main(String[] array) throws Exception {
        int n = 0;
        int parsingOptions = 2;
        int n2 = 1;
        if (array.length < 1 || array.length > 2) {
            n2 = 0;
        }
        if (n2 != 0 && "-debug".equals(array[0])) {
            n = 1;
            parsingOptions = 0;
            if (array.length != 2) {
                n2 = 0;
            }
        }
        if (n2 == 0) {
            System.err.println("Prints the ASM code to generate the given class.");
            System.err.println("Usage: ASMifier [-debug] <fully qualified class name or class file name>");
            return;
        }
        Object o;
        if (array[n].endsWith(".class") || array[n].indexOf(92) > -1 || array[n].indexOf(47) > -1) {
            o = new ClassReader(new FileInputStream(array[n]));
        }
        else {
            o = new ClassReader(array[n]);
        }
        o.accept(new TraceClassVisitor(null, new ASMifier(), new PrintWriter(System.out)), parsingOptions);
        return;
    }
    
    @Override
    public void visit(int i, int n, String substring, String s, String s2, String[] array) {
        int lastIndex = substring.lastIndexOf(47);
        if (lastIndex == -1) {
        }
        else {
            this.text.add("package asm." + substring.substring(0, lastIndex).replace('/', '.') + ";\n");
            substring = substring.substring(lastIndex + 1);
        }
        this.text.add("import java.util.*;\n");
        this.text.add("import jdk.internal.org.objectweb.asm.*;\n");
        this.text.add("public class " + substring + "Dump implements Opcodes {\n\n");
        this.text.add("public static byte[] dump () throws Exception {\n\n");
        this.text.add("ClassWriter cw = new ClassWriter(0);\n");
        this.text.add("FieldVisitor fv;\n");
        this.text.add("MethodVisitor mv;\n");
        this.text.add("AnnotationVisitor av0;\n\n");
        this.buf.setLength(0);
        this.buf.append("cw.visit(");
        switch (i) {
            case 196653: {
                this.buf.append("V1_1");
                break;
            }
            case 46: {
                this.buf.append("V1_2");
                break;
            }
            case 47: {
                this.buf.append("V1_3");
                break;
            }
            case 48: {
                this.buf.append("V1_4");
                break;
            }
            case 49: {
                this.buf.append("V1_5");
                break;
            }
            case 50: {
                this.buf.append("V1_6");
                break;
            }
            case 51: {
                this.buf.append("V1_7");
                break;
            }
            default: {
                this.buf.append(i);
                break;
            }
        }
        this.buf.append(", ");
        this.appendAccess(n | 0x40000);
        this.buf.append(", ");
        this.appendConstant(substring);
        this.buf.append(", ");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        if (array != null && array.length > 0) {
            this.buf.append("new String[] {");
            for (int j = 0; j < array.length; ++j) {
                this.buf.append((j == 0) ? " " : ", ");
                this.appendConstant(array[j]);
            }
            this.buf.append(" }");
        }
        else {
            this.buf.append("null");
        }
        this.buf.append(");\n\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitSource(String s, String s2) {
        this.buf.setLength(0);
        this.buf.append("cw.visitSource(");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(");\n\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitOuterClass(String s, String s2, String s3) {
        this.buf.setLength(0);
        this.buf.append("cw.visitOuterClass(");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(s3);
        this.buf.append(");\n\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public ASMifier visitClassAnnotation(String s, boolean b) {
        return this.visitAnnotation(s, b);
    }
    
    @Override
    public ASMifier visitClassTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation(n, typePath, s, b);
    }
    
    @Override
    public void visitClassAttribute(Attribute attribute) {
        this.visitAttribute(attribute);
    }
    
    @Override
    public void visitInnerClass(String s, String s2, String s3, int n) {
        this.buf.setLength(0);
        this.buf.append("cw.visitInnerClass(");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(s3);
        this.buf.append(", ");
        this.appendAccess(n | 0x100000);
        this.buf.append(");\n\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public ASMifier visitField(int n, String s, String s2, String s3, Object o) {
        this.buf.setLength(0);
        this.buf.append("{\n");
        this.buf.append("fv = cw.visitField(");
        this.appendAccess(n | 0x80000);
        this.buf.append(", ");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(s3);
        this.buf.append(", ");
        this.appendConstant(o);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("fv", 0);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    @Override
    public ASMifier visitMethod(int n, String s, String s2, String s3, String[] array) {
        this.buf.setLength(0);
        this.buf.append("{\n");
        this.buf.append("mv = cw.visitMethod(");
        this.appendAccess(n);
        this.buf.append(", ");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(s3);
        this.buf.append(", ");
        if (array != null && array.length > 0) {
            this.buf.append("new String[] {");
            for (int i = 0; i < array.length; ++i) {
                this.buf.append((i == 0) ? " " : ", ");
                this.appendConstant(array[i]);
            }
            this.buf.append(" }");
        }
        else {
            this.buf.append("null");
        }
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("mv", 0);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    @Override
    public void visitClassEnd() {
        this.text.add("cw.visitEnd();\n\n");
        this.text.add("return cw.toByteArray();\n");
        this.text.add("}\n");
        this.text.add("}\n");
    }
    
    @Override
    public void visit(String s, Object o) {
        this.buf.setLength(0);
        this.buf.append("av").append(this.id).append(".visit(");
        appendConstant(this.buf, s);
        this.buf.append(", ");
        appendConstant(this.buf, o);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitEnum(String s, String s2, String s3) {
        this.buf.setLength(0);
        this.buf.append("av").append(this.id).append(".visitEnum(");
        appendConstant(this.buf, s);
        this.buf.append(", ");
        appendConstant(this.buf, s2);
        this.buf.append(", ");
        appendConstant(this.buf, s3);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public ASMifier visitAnnotation(String s, String s2) {
        this.buf.setLength(0);
        this.buf.append("{\n");
        this.buf.append("AnnotationVisitor av").append(this.id + 1).append(" = av");
        this.buf.append(this.id).append(".visitAnnotation(");
        appendConstant(this.buf, s);
        this.buf.append(", ");
        appendConstant(this.buf, s2);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("av", this.id + 1);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    @Override
    public ASMifier visitArray(String s) {
        this.buf.setLength(0);
        this.buf.append("{\n");
        this.buf.append("AnnotationVisitor av").append(this.id + 1).append(" = av");
        this.buf.append(this.id).append(".visitArray(");
        appendConstant(this.buf, s);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("av", this.id + 1);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    @Override
    public void visitAnnotationEnd() {
        this.buf.setLength(0);
        this.buf.append("av").append(this.id).append(".visitEnd();\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public ASMifier visitFieldAnnotation(String s, boolean b) {
        return this.visitAnnotation(s, b);
    }
    
    @Override
    public ASMifier visitFieldTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation(n, typePath, s, b);
    }
    
    @Override
    public void visitFieldAttribute(Attribute attribute) {
        this.visitAttribute(attribute);
    }
    
    @Override
    public void visitFieldEnd() {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitEnd();\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitParameter(String s, int n) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitParameter(");
        Printer.appendString(this.buf, s);
        this.buf.append(", ");
        this.appendAccess(n);
        this.text.add(this.buf.append(");\n").toString());
    }
    
    @Override
    public ASMifier visitAnnotationDefault() {
        this.buf.setLength(0);
        this.buf.append("{\n").append("av0 = ").append(this.name).append(".visitAnnotationDefault();\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("av", 0);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    @Override
    public ASMifier visitMethodAnnotation(String s, boolean b) {
        return this.visitAnnotation(s, b);
    }
    
    @Override
    public ASMifier visitMethodTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation(n, typePath, s, b);
    }
    
    @Override
    public ASMifier visitParameterAnnotation(int i, String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append("{\n").append("av0 = ").append(this.name).append(".visitParameterAnnotation(").append(i).append(", ");
        this.appendConstant(s);
        this.buf.append(", ").append(b).append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("av", 0);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    @Override
    public void visitMethodAttribute(Attribute attribute) {
        this.visitAttribute(attribute);
    }
    
    @Override
    public void visitCode() {
        this.text.add(this.name + ".visitCode();\n");
    }
    
    @Override
    public void visitFrame(int n, int i, Object[] array, int j, Object[] array2) {
        this.buf.setLength(0);
        switch (n) {
            case -1:
            case 0: {
                this.declareFrameTypes(i, array);
                this.declareFrameTypes(j, array2);
                if (n == -1) {
                    this.buf.append(this.name).append(".visitFrame(Opcodes.F_NEW, ");
                }
                else {
                    this.buf.append(this.name).append(".visitFrame(Opcodes.F_FULL, ");
                }
                this.buf.append(i).append(", new Object[] {");
                this.appendFrameTypes(i, array);
                this.buf.append("}, ").append(j).append(", new Object[] {");
                this.appendFrameTypes(j, array2);
                this.buf.append('}');
                break;
            }
            case 1: {
                this.declareFrameTypes(i, array);
                this.buf.append(this.name).append(".visitFrame(Opcodes.F_APPEND,").append(i).append(", new Object[] {");
                this.appendFrameTypes(i, array);
                this.buf.append("}, 0, null");
                break;
            }
            case 2: {
                this.buf.append(this.name).append(".visitFrame(Opcodes.F_CHOP,").append(i).append(", null, 0, null");
                break;
            }
            case 3: {
                this.buf.append(this.name).append(".visitFrame(Opcodes.F_SAME, 0, null, 0, null");
                break;
            }
            case 4: {
                this.declareFrameTypes(1, array2);
                this.buf.append(this.name).append(".visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {");
                this.appendFrameTypes(1, array2);
                this.buf.append('}');
                break;
            }
        }
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitInsn(int n) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitInsn(").append(ASMifier.OPCODES[n]).append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitIntInsn(int n, int i) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitIntInsn(").append(ASMifier.OPCODES[n]).append(", ").append((n == 188) ? ASMifier.TYPES[i] : Integer.toString(i)).append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitVarInsn(int n, int i) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitVarInsn(").append(ASMifier.OPCODES[n]).append(", ").append(i).append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitTypeInsn(int n, String s) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitTypeInsn(").append(ASMifier.OPCODES[n]).append(", ");
        this.appendConstant(s);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitFieldInsn(int n, String s, String s2, String s3) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitFieldInsn(").append(ASMifier.OPCODES[n]).append(", ");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(s3);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int n, String s, String s2, String s3) {
        if (this.api >= 327680) {
            super.visitMethodInsn(n, s, s2, s3);
            return;
        }
        this.doVisitMethodInsn(n, s, s2, s3, n == 185);
        return;
    }
    
    @Override
    public void visitMethodInsn(int n, String s, String s2, String s3, boolean b) {
        if (this.api < 327680) {
            super.visitMethodInsn(n, s, s2, s3, b);
            return;
        }
        this.doVisitMethodInsn(n, s, s2, s3, b);
        return;
    }
    
    private void doVisitMethodInsn(int n, String s, String s2, String s3, boolean b) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitMethodInsn(").append(ASMifier.OPCODES[n]).append(", ");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(s3);
        this.buf.append(", ");
        this.buf.append(b ? "true" : "false");
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitInvokeDynamicInsn(String s, String s2, Handle handle, Object... array) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitInvokeDynamicInsn(");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(handle);
        this.buf.append(", new Object[]{");
        for (int i = 0; i < array.length; ++i) {
            this.appendConstant(array[i]);
            if (i != array.length - 1) {
                this.buf.append(", ");
            }
        }
        this.buf.append("});\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitJumpInsn(int n, Label label) {
        this.buf.setLength(0);
        this.declareLabel(label);
        this.buf.append(this.name).append(".visitJumpInsn(").append(ASMifier.OPCODES[n]).append(", ");
        this.appendLabel(label);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitLabel(Label label) {
        this.buf.setLength(0);
        this.declareLabel(label);
        this.buf.append(this.name).append(".visitLabel(");
        this.appendLabel(label);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitLdcInsn(Object o) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitLdcInsn(");
        this.appendConstant(o);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitIincInsn(int i, int j) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitIincInsn(").append(i).append(", ").append(j).append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitTableSwitchInsn(int i, int j, Label label, Label... array) {
        this.buf.setLength(0);
        for (int k = 0; k < array.length; ++k) {
            this.declareLabel(array[k]);
        }
        this.declareLabel(label);
        this.buf.append(this.name).append(".visitTableSwitchInsn(").append(i).append(", ").append(j).append(", ");
        this.appendLabel(label);
        this.buf.append(", new Label[] {");
        for (int l = 0; l < array.length; ++l) {
            this.buf.append((l == 0) ? " " : ", ");
            this.appendLabel(array[l]);
        }
        this.buf.append(" });\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitLookupSwitchInsn(Label label, int[] array, Label[] array2) {
        this.buf.setLength(0);
        for (int i = 0; i < array2.length; ++i) {
            this.declareLabel(array2[i]);
        }
        this.declareLabel(label);
        this.buf.append(this.name).append(".visitLookupSwitchInsn(");
        this.appendLabel(label);
        this.buf.append(", new int[] {");
        for (int j = 0; j < array.length; ++j) {
            this.buf.append((j == 0) ? " " : ", ").append(array[j]);
        }
        this.buf.append(" }, new Label[] {");
        for (int k = 0; k < array2.length; ++k) {
            this.buf.append((k == 0) ? " " : ", ");
            this.appendLabel(array2[k]);
        }
        this.buf.append(" });\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitMultiANewArrayInsn(String s, int i) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitMultiANewArrayInsn(");
        this.appendConstant(s);
        this.buf.append(", ").append(i).append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public ASMifier visitInsnAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation("visitInsnAnnotation", n, typePath, s, b);
    }
    
    @Override
    public void visitTryCatchBlock(Label label, Label label2, Label label3, String s) {
        this.buf.setLength(0);
        this.declareLabel(label);
        this.declareLabel(label2);
        this.declareLabel(label3);
        this.buf.append(this.name).append(".visitTryCatchBlock(");
        this.appendLabel(label);
        this.buf.append(", ");
        this.appendLabel(label2);
        this.buf.append(", ");
        this.appendLabel(label3);
        this.buf.append(", ");
        this.appendConstant(s);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public ASMifier visitTryCatchAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation("visitTryCatchAnnotation", n, typePath, s, b);
    }
    
    @Override
    public void visitLocalVariable(String s, String s2, String s3, Label label, Label label2, int i) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitLocalVariable(");
        this.appendConstant(s);
        this.buf.append(", ");
        this.appendConstant(s2);
        this.buf.append(", ");
        this.appendConstant(s3);
        this.buf.append(", ");
        this.appendLabel(label);
        this.buf.append(", ");
        this.appendLabel(label2);
        this.buf.append(", ").append(i).append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Printer visitLocalVariableAnnotation(int i, TypePath obj, Label[] array, Label[] array2, int[] array3, String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append("{\n").append("av0 = ").append(this.name).append(".visitLocalVariableAnnotation(");
        this.buf.append(i);
        this.buf.append(", TypePath.fromString(\"").append(obj).append("\"), ");
        this.buf.append("new Label[] {");
        for (int j = 0; j < array.length; ++j) {
            this.buf.append((j == 0) ? " " : ", ");
            this.appendLabel(array[j]);
        }
        this.buf.append(" }, new Label[] {");
        for (int k = 0; k < array2.length; ++k) {
            this.buf.append((k == 0) ? " " : ", ");
            this.appendLabel(array2[k]);
        }
        this.buf.append(" }, new int[] {");
        for (int l = 0; l < array3.length; ++l) {
            this.buf.append((l == 0) ? " " : ", ").append(array3[l]);
        }
        this.buf.append(" }, ");
        this.appendConstant(s);
        this.buf.append(", ").append(b).append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("av", 0);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    @Override
    public void visitLineNumber(int i, Label label) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitLineNumber(").append(i).append(", ");
        this.appendLabel(label);
        this.buf.append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitMaxs(int i, int j) {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitMaxs(").append(i).append(", ").append(j).append(");\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitMethodEnd() {
        this.buf.setLength(0);
        this.buf.append(this.name).append(".visitEnd();\n");
        this.text.add(this.buf.toString());
    }
    
    public ASMifier visitAnnotation(String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append("{\n").append("av0 = ").append(this.name).append(".visitAnnotation(");
        this.appendConstant(s);
        this.buf.append(", ").append(b).append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("av", 0);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    public ASMifier visitTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation("visitTypeAnnotation", n, typePath, s, b);
    }
    
    public ASMifier visitTypeAnnotation(String str, int i, TypePath obj, String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append("{\n").append("av0 = ").append(this.name).append(".").append(str).append("(");
        this.buf.append(i);
        this.buf.append(", TypePath.fromString(\"").append(obj).append("\"), ");
        this.appendConstant(s);
        this.buf.append(", ").append(b).append(");\n");
        this.text.add(this.buf.toString());
        ASMifier asMifier = this.createASMifier("av", 0);
        this.text.add(asMifier.getText());
        this.text.add("}\n");
        return asMifier;
    }
    
    public void visitAttribute(Attribute attribute) {
        this.buf.setLength(0);
        this.buf.append("// ATTRIBUTE ").append(attribute.type).append('\n');
        if ((attribute instanceof ASMifiable)) {
            if (this.labelNames == null) {
                this.labelNames = new HashMap();
            }
            this.buf.append("{\n");
            ((ASMifiable)attribute).asmify(this.buf, "attr", this.labelNames);
            this.buf.append(this.name).append(".visitAttribute(attr);\n");
            this.buf.append("}\n");
        }
        this.text.add(this.buf.toString());
    }
    
    protected ASMifier createASMifier(String s, int n) {
        return new ASMifier(327680, s, n);
    }
    
    void appendAccess(int n) {
        int n2 = 1;
        if ((n & 0x1) != 0x0) {
            this.buf.append("ACC_PUBLIC");
            n2 = 0;
        }
        if ((n & 0x2) != 0x0) {
            this.buf.append("ACC_PRIVATE");
            n2 = 0;
        }
        if ((n & 0x4) != 0x0) {
            this.buf.append("ACC_PROTECTED");
            n2 = 0;
        }
        if ((n & 0x10) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_FINAL");
            n2 = 0;
        }
        if ((n & 0x8) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_STATIC");
            n2 = 0;
        }
        if ((n & 0x20) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            if ((n & 0x40000) == 0x0) {
                this.buf.append("ACC_SYNCHRONIZED");
            }
            else {
                this.buf.append("ACC_SUPER");
            }
            n2 = 0;
        }
        if ((n & 0x40) != 0x0 && (n & 0x80000) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_VOLATILE");
            n2 = 0;
        }
        if ((n & 0x40) != 0x0 && (n & 0x40000) == 0x0 && (n & 0x80000) == 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_BRIDGE");
            n2 = 0;
        }
        if ((n & 0x80) != 0x0 && (n & 0x40000) == 0x0 && (n & 0x80000) == 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_VARARGS");
            n2 = 0;
        }
        if ((n & 0x80) != 0x0 && (n & 0x80000) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_TRANSIENT");
            n2 = 0;
        }
        if ((n & 0x100) != 0x0 && (n & 0x40000) == 0x0 && (n & 0x80000) == 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_NATIVE");
            n2 = 0;
        }
        if ((n & 0x4000) != 0x0 && ((n & 0x40000) != 0x0 || (n & 0x80000) != 0x0 || (n & 0x100000) != 0x0)) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_ENUM");
            n2 = 0;
        }
        if ((n & 0x2000) != 0x0 && ((n & 0x40000) != 0x0 || (n & 0x100000) != 0x0)) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_ANNOTATION");
            n2 = 0;
        }
        if ((n & 0x400) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_ABSTRACT");
            n2 = 0;
        }
        if ((n & 0x200) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_INTERFACE");
            n2 = 0;
        }
        if ((n & 0x800) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_STRICT");
            n2 = 0;
        }
        if ((n & 0x1000) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_SYNTHETIC");
            n2 = 0;
        }
        if ((n & 0x20000) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_DEPRECATED");
            n2 = 0;
        }
        if ((n & 0x8000) != 0x0) {
            if (n2 == 0) {
                this.buf.append(" + ");
            }
            this.buf.append("ACC_MANDATED");
            n2 = 0;
        }
        if (n2 != 0) {
            this.buf.append('0');
        }
    }
    
    protected void appendConstant(Object o) {
        appendConstant(this.buf, o);
    }
    
    static void appendConstant(StringBuffer sb, Object o) {
        if (o == null) {
            sb.append("null");
        }
        else if ((o instanceof String)) {
            Printer.appendString(sb, (String)o);
        }
        else if ((o instanceof Type)) {
            sb.append("Type.getType(\"");
            sb.append(((Type)o).getDescriptor());
            sb.append("\")");
        }
        else if ((o instanceof Handle)) {
            sb.append("new Handle(");
            Handle handle = (Handle)o;
            sb.append("Opcodes.").append(ASMifier.HANDLE_TAG[handle.getTag()]).append(", \"");
            sb.append(handle.getOwner()).append("\", \"");
            sb.append(handle.getName()).append("\", \"");
            sb.append(handle.getDesc()).append("\")");
        }
        else if ((o instanceof Byte)) {
            sb.append("new Byte((byte)").append(o).append(')');
        }
        else if ((o instanceof Boolean)) {
            sb.append(((boolean)o) ? "Boolean.TRUE" : "Boolean.FALSE");
        }
        else if ((o instanceof Short)) {
            sb.append("new Short((short)").append(o).append(')');
        }
        else if ((o instanceof Character)) {
            sb.append("new Character((char)").append((int)(char)o).append(')');
        }
        else if ((o instanceof Integer)) {
            sb.append("new Integer(").append(o).append(')');
        }
        else if ((o instanceof Float)) {
            sb.append("new Float(\"").append(o).append("\")");
        }
        else if ((o instanceof Long)) {
            sb.append("new Long(").append(o).append("L)");
        }
        else if ((o instanceof Double)) {
            sb.append("new Double(\"").append(o).append("\")");
        }
        else if ((o instanceof byte[])) {
            byte[] array8 = (byte[])(byte[])o;
            sb.append("new byte[] {");
            for (int n4 = 0; n4 < array8.length; ++n4) {
                sb.append((n4 == 0) ? "" : ",").append((int)array8[n4]);
            }
            sb.append('}');
        }
        else if ((o instanceof boolean[])) {
            boolean[] array7 = (boolean[])(boolean[])o;
            sb.append("new boolean[] {");
            for (int n3 = 0; n3 < array7.length; ++n3) {
                sb.append((n3 == 0) ? "" : ",").append(array7[n3]);
            }
            sb.append('}');
        }
        else if ((o instanceof short[])) {
            short[] array6 = (short[])(short[])o;
            sb.append("new short[] {");
            for (int n2 = 0; n2 < array6.length; ++n2) {
                sb.append((n2 == 0) ? "" : ",").append("(short)").append((int)array6[n2]);
            }
            sb.append('}');
        }
        else if ((o instanceof char[])) {
            char[] array5 = (char[])(char[])o;
            sb.append("new char[] {");
            for (int n = 0; n < array5.length; ++n) {
                sb.append((n == 0) ? "" : ",").append("(char)").append((int)array5[n]);
            }
            sb.append('}');
        }
        else if ((o instanceof int[])) {
            int[] array4 = (int[])(int[])o;
            sb.append("new int[] {");
            for (int l = 0; l < array4.length; ++l) {
                sb.append((l == 0) ? "" : ",").append(array4[l]);
            }
            sb.append('}');
        }
        else if ((o instanceof long[])) {
            long[] array3 = (long[])(long[])o;
            sb.append("new long[] {");
            for (int k = 0; k < array3.length; ++k) {
                sb.append((k == 0) ? "" : ",").append(array3[k]).append('L');
            }
            sb.append('}');
        }
        else if ((o instanceof float[])) {
            float[] array2 = (float[])(float[])o;
            sb.append("new float[] {");
            for (int j = 0; j < array2.length; ++j) {
                sb.append((j == 0) ? "" : ",").append(array2[j]).append('f');
            }
            sb.append('}');
        }
        else if ((o instanceof double[])) {
            double[] array = (double[])(double[])o;
            sb.append("new double[] {");
            for (int i = 0; i < array.length; ++i) {
                sb.append((i == 0) ? "" : ",").append(array[i]).append('d');
            }
            sb.append('}');
        }
    }
    
    private void declareFrameTypes(int n, Object[] array) {
        for (int i = 0; i < n; ++i) {
            if ((array[i] instanceof Label)) {
                this.declareLabel((Label)array[i]);
            }
        }
    }
    
    private void appendFrameTypes(int n, Object[] array) {
        for (int i = 0; i < n; ++i) {
            if (i > 0) {
                this.buf.append(", ");
            }
            if ((array[i] instanceof String)) {
                this.appendConstant(array[i]);
            }
            else if ((array[i] instanceof Integer)) {
                switch ((int)array[i]) {
                    case 0: {
                        this.buf.append("Opcodes.TOP");
                        break;
                    }
                    case 1: {
                        this.buf.append("Opcodes.INTEGER");
                        break;
                    }
                    case 2: {
                        this.buf.append("Opcodes.FLOAT");
                        break;
                    }
                    case 3: {
                        this.buf.append("Opcodes.DOUBLE");
                        break;
                    }
                    case 4: {
                        this.buf.append("Opcodes.LONG");
                        break;
                    }
                    case 5: {
                        this.buf.append("Opcodes.NULL");
                        break;
                    }
                    case 6: {
                        this.buf.append("Opcodes.UNINITIALIZED_THIS");
                        break;
                    }
                }
            }
            else {
                this.appendLabel((Label)array[i]);
            }
        }
    }
    
    protected void declareLabel(Label label) {
        if (this.labelNames == null) {
            this.labelNames = new HashMap();
        }
        if ((String)this.labelNames.get(label) == null) {
            String string = "l" + this.labelNames.size();
            this.labelNames.put(label, string);
            this.buf.append("Label ").append(string).append(" = new Label();\n");
        }
    }
    
    protected void appendLabel(Label label) {
        this.buf.append((String)this.labelNames.get(label));
    }
}
